/**
 * Valencia Renderer - realtime Mandelbulb demo
 * Copyright (C) 2013 Adam Celarek (www.celarek.at)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef CONSTANTS_H
#define CONSTANTS_H

unsigned const int RENDER_WIDTH = 960;
unsigned const int RENDER_HEIGHT = 960; //must be divisable through ACCELERATION_BLOCK_SIZE and KERNEL_WIDTH for the new algorithm.

// speed (threads per block): 4x4 < 32x32 < 8x8 = 16x16 < 8x16
unsigned const int THREADS_PER_BLOCK_WIDTH = 8;
unsigned const int THREADS_PER_BLOCK_HEIGHT = 16;


//#define ACCELERATION_ACCURATE_RAY_BLOCK
unsigned const int ACCELERATION_BLOCK_SIZE = 3; // 3, 5 or 7. higher numbers would need a change in the acceleration code.
#ifdef ACCELERATION_ACCURATE_RAY_BLOCK
unsigned const int ACCELERATION_RAY_COUNT = ACCELERATION_BLOCK_SIZE*ACCELERATION_BLOCK_SIZE;
unsigned const int ACCELERATION_STRUCT_MIDDLE_RAY = ACCELERATION_RAY_COUNT / 2;
#else
unsigned const int ACCELERATION_RAY_COUNT = 3;  // 3, 6 or 10 (corresponding)
unsigned const int ACCELERATION_STRUCT_MIDDLE_RAY = 0;
#endif

#define VALENCIA_INLINE __forceinline__

// from "Ray tracing deterministic 3-D fractals" by Hart et al
#define FRACTAL_EPSILON_ALPHA 0.001f
#define SHADOW_EPSILON_ALPHA 0.05f

#define FRACTAL_MAX_ITERATIONS 10
#define FRACTAL_MAX_RAY_STEPS 50

// less important:
#define RAY_MAX_DISTANCE 4.0f
//FRACTAL_Z_LENGTH_BAILOUT 1.30! it doesn't change much, if it is higher, only fps because of iteration count.
//-> FRACTAL_Z_LENGTH_BAILOUT 1.20 the fractal gets damaged, 1.24 should work, 1.30 is safe. (empirically tested)
#define FRACTAL_Z_LENGTH_BAILOUT 1.30f

#endif
