/**
 * Valencia Renderer - realtime Mandelbulb demo
 * Copyright (C) 2013 Adam Celarek (www.celarek.at)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef NORMAL_ESTIMATOR_H
#define NORMAL_ESTIMATOR_H

#include "DoubleMath.h"

namespace valencia {

template<typename SignedDistance>
static __device__
myFloat3 grad(SignedDistance d, const myFloat3 &x, const myFloat eps)
{
	/* * /
	myFloat dx = d(x + make_myFloat3(eps,    0,   0)) - d(x - make_myFloat3(eps,   0,   0));
	myFloat dy = d(x + make_myFloat3(  0,  eps,   0)) - d(x - make_myFloat3(  0, eps,   0));
	myFloat dz = d(x + make_myFloat3(  0,    0, eps)) - d(x - make_myFloat3(  0,   0, eps));
    /* */
	/* */
	myFloat xDist = d(x);
	myFloat 
	dx = d(x + make_myFloat3(eps,    0,   0)) - xDist;
	myFloat
	dy = d(x + make_myFloat3(  0,  eps,   0)) - xDist;
	myFloat
	dz = d(x + make_myFloat3(  0,    0, eps)) - xDist;
	/* */

	return make_myFloat3(dx, dy, dz);
}


template<typename SignedDistance>
static __device__
myFloat3 estimate_normal(const SignedDistance distance,
                       const myFloat3 &x,
                       const myFloat epsilon = 2e-6f)
{
  return normalize(grad(distance, x, epsilon));
}

}

#endif