/**
 * Valencia Renderer - realtime Mandelbulb demo
 * Copyright (C) 2013 Adam Celarek (www.celarek.at)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef VALENCIA_RENDERER_CUH
#define VALENCIA_RENDERER_CUH

#include <helper_math.h>
#include <helper_functions.h>
#include "PinholeCamera.h"


__constant__ PinholeCamera::CameraData cameraData[1];

struct RenderParameters {
	bool shadow;
	char fractalMaxIterations;
	short maxMarchingSteps;
	float fractalEpsilonAlpha;
	float shadowEpsilonAlpha;
};
__constant__ RenderParameters renderParams[1];

enum Direction {Increase, Decrease};

class ValenciaRenderer {
private:
	PinholeCamera m_camera;
	int m_width, m_height;

	float m_deltaTime;
	StopWatchInterface* m_timer;

	bool m_acceleratedRenderMode;
	RenderParameters m_renderParams;

	void updateCamera();
	void updateRenderParam();
public:
	ValenciaRenderer(int width, int height);
	~ValenciaRenderer();
	double render(unsigned int *d_dest, StopWatchInterface *timer);

	void moveForward();
	void moveBackward();
	void rotateCamera(float dx, float dy);
	void printInfos();
	bool toggleAcceleratedRenderMode() { m_acceleratedRenderMode = !m_acceleratedRenderMode; return m_acceleratedRenderMode; }
	bool toggleShadows();
	char changFractalMaxIterations(Direction dir);
	short changMaxMarchingSteps(Direction dir);
	float changFractalEpsilonAlpha(Direction dir);
	float changShadowEpsilonAlpha(Direction dir);

};

#endif